
ConfigTable::DESCRIPTER.concat [

  [ 'with',     [ '',
                  'name,name...',
                  'package names that you want to install',
                  'ALL' ] ],
  [ 'without',  [ '',
                  'name,name...',
                  'package names that you do not want to install',
                  'NONE' ] ]
]


class PackageManager < ToplevelInstaller

  def initialize( root )
    super
    @packages = nil
  end

  def run_metaconfigs
    eval_file_ifexist "#{srcdir_root()}/metaconfig"
    packages().each do |name|
      eval_file_ifexist "#{srcdir_root()}/packages/#{name}/metaconfig"
    end
  end

  def print_usage( f )
    super
    f.puts 'Inluded packages:'
    f.puts '  ' + packages.sort.join(' ')
    f.puts
  end

  #
  # setup.rb metaconfig API
  #

  def declare_packages( list )
    list.each do |name|
      raise InstallError, "directory packages/#{name} does not exist"\
              unless dir?("#{srcdir_root()}/packages/#{name}")
    end
    @packages = list
  end

  #
  # packages
  #

  def packages
    @packages ||= all_dirs_in(srcdir + '/packages')
    raise InstallError, 'no package available' if @packages.empty?
    @packages
  end

  def load_installers
    @installers = {}
    packages.each do |dir|
      @installers[dir] = Installer.new(@config, @options,
                                       srcdir + '/packages/' + dir,
                                       'packages/' + dir)
    end
  end

  def init_selection
    with    = extract_selection(config('with'))
    without = extract_selection(config('without'))

    @selected = @installers.keys.select {|name|
        (with.empty? or with.include? name) and
        not without.include? name
    }
  end

  def extract_selection( list )
    a = list.split(/,/)
    a.each do |name|
      raise InstallError, "no such package: #{name}" unless @installers.key? name
    end
    a
  end

  def each_selected_installers
    dive_into('packages') {
        @selected.each do |name|
          dive_into('packages/' + name) {
              yield @installers[name]
          }
        end
    }
  end

  #
  # tasks
  #

  def exectask( task )
    load_installers
    init_selection
    super
  end

  def exec_config
    run_hook 'pre-config'
    each_selected_installers {|inst| inst.exec_config }
    @config.save
    run_hook 'post-config'
  end

  def exec_setup
    run_hook 'pre-setup'
    each_selected_installers {|inst| inst.exec_setup }
    run_hook 'post-setup'
  end

  def exec_install
    run_hook 'pre-install'
    each_selected_installers {|inst| inst.exec_install }
    run_hook 'post-install'
  end

  def exec_clean
    run_hook 'pre-clean'
    each_selected_installers {|inst| inst.exec_clean }
    run_hook 'post-clean'
  end

  def exec_distclean
    run_hook 'pre-distclean'
    each_selected_installers {|inst| inst.exec_distclean }
    run_hook 'post-distclean'
  end

end

